<?php
/*
Plugin Name: Latest Tweet
Plugin URI: http://www.stormyfrog.com/wordpress/latest-tweet/
Description: Display the most recent tweet from your unprotected twitter timeline with a simple caching system and no JavaScript (server fetches tweets).
Version: 1.1.0
Author: Andrew Buckman
Author URI: http://www.abuckman.com/

Copyright 2009-2010 Andrew Buckman (andrewb@stormyfrog.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

if (!class_exists('latest_tweet')) {
	class latest_tweet {
		
		function latest_tweet() {
			$this->version = '1.1.0';
		}
		
		function get_settings() {
			# Set up defaults
			$settings = array(
				# Debug mode
				'debug' => false,
				# How frequently to update cache? [hourly|twicedaily|daily]
				'cache_update' => 'hourly',
				# Twitter screen name
				'screen_name' => '',
				# The number of tweets you want to fetch
				# * get more than you need so you can exclude replys
				'tweets_to_check' => '10',
				# {old} Cache expiration in seconds
				'expires' => '60',
				# the code to print out for the tweet
				# meta tags available:
				# '%screen_name%','%followers%','%tweet_id%','%tweet_raw%','%tweet%','%created_at%'
				# '%rel_time%' -- calculated value, time since created_at
				'html_template' => '
<blockquote class="tweet">
	%tweet%
	<a href="http://twitter.com/%screen_name%/statuses/%tweet_id%" class="reltime" onclick="pageTracker._trackPageview(\'/outbound/twitter/reltime\');">%rel_time%</a>
	<cite><a href="http://twitter.com/%screen_name%" class="followers" onclick="pageTracker._trackPageview(\'/outbound/twitter/followers\');">Join me and %followers% followers on twitter.</a></cite>
</blockquote>	
'
			);
			if ( get_option('latest_tweet_settings') )
				$settings = array_merge( $settings, get_option('latest_tweet_settings') );
			return $settings;
		}
		
		function twitter_link( $text ) {
			$ret = preg_replace( "/((https?|s?ftp|ssh)\:\/\/[^\"\s\<\>]*[^.,;'\">\:\s\<\>\)\]\!])/ie", "'<a href=\"\\1\">\\1</a>'", $text );
			$ret = preg_replace( "/\B@([_a-z0-9]+)/ie", "'@<a href=\"http://twitter.com/\\1\">\\1</a>'", $ret );
			return $ret;
		}
		
		function relative_time( $time_value ) {
			/* NOTE: There is a WP function for this already: human_time_diff( $from, {$to} ) */
			$parsed_date = strtotime( $time_value );
			$relative_to = time();
			$delta = $relative_to - $parsed_date;

			if ( $delta < 60 ) {
				return 'less than a minute ago';
			} else if( $delta < 120 ) {
				return 'about a minute ago';
			} else if( $delta < (60*60) ) {
				return round($delta / 60) . ' minutes ago';
			} else if( $delta < (120*60) ) {
				return 'about an hour ago';
			} else if( $delta < (24*60*60) ) {
				return 'about ' . round($delta / 3600) . ' hours ago';
			} else if( $delta < (48*60*60) ) {
				return '1 day ago';
			} else {
				return round( $delta / 86400 ) . ' days ago';
			}
		}
		
		function get_cache() {
			# Set up default
			$cache = array();

			# Merge cached data
			if ( get_option('latest_tweet_cache') )
				$cache = array_merge( $cache, get_option('latest_tweet_cache') );
			# echo 'cache:'; print_r($cache);
			return $cache;
		}
		
		function cache_tweets( $settings = array() ) {
			$settings = array_merge( $this->get_settings(), $settings );
			$cache = $this->get_cache();
			
			# verify screen name specified, cowardly refuse to cache otherwise
			if ( !$settings['screen_name'] || trim($settings['screen_name']) == '' ) return false;
			
			# do the work, fetch latest, bla bla
			include_once( 'twitter.api.php' );
			try {
				$twitter = new Twitter;
				$entries = $twitter->statuses->user_timeline( 
					array(
						'screen_name' => $settings['screen_name'], 
						'count' => $settings['tweets_to_check']
					)
				);
			} catch ( TwitterResponseException $e ) {
				if ( $settings['debug'] ) echo 'EXCEPTION: ' . $e->getMessage();
				if ( $settings['debug'] ) print_r( $response );
				#!! Instill delay for Error Code 503
				
				# update timestamp
				$upd = get_option( 'latest_tweet_settings' );
				$upd['last_updated'] = time();
				update_option( 'latest_tweet_settings', $upd );
				return false;
			}

			if ( $settings['debug'] ) {
				echo 'ENTRIES: ';
				print_r( $entries );
			}
			if ( is_array($entries) ) {
				# Search through {tweets_to_check} latest tweets to find newest non-reply
				foreach ( $entries as $tweet ) {
					if ( !$tweet->in_reply_to_user_id ) {
						$lastTweet = $tweet;
						break;
					}
				}

				# If all tweets were replies, default to most recent tweet
				if ( !$lastTweet ) $lastTweet = $entries[0];

				# Build Cache Data
				if ( $lastTweet->text > '' ) {
					$cache = array(
						'screen_name' => $lastTweet->user->screen_name,
						'followers' => $lastTweet->user->followers_count,
						'tweet_id' => $lastTweet->id,
						'tweet_raw' => $lastTweet->text,
						'tweet' => $this->twitter_link( $lastTweet->text ),
						'created_at' => $lastTweet->created_at
					);
					
					# save updated cache data
					update_option( 'latest_tweet_cache', $cache );
				}
			}

			# update timestamp
			$upd = get_option( 'latest_tweet_settings' );
			$upd['last_updated'] = time();
			update_option( 'latest_tweet_settings', $upd );
			
			return true;
		}
		
		function print_tweet( $settings = array(), $output_results = true ) {
			$settings = array_merge( $this->get_settings(), $settings );
			$cache = $this->get_cache();
			if ( $settings['debug'] ) {
				echo 'SETTINGS: ';
				print_r( $settings );
				echo 'CACHE: ';
				print_r( $cache );
			}
			
			# Verify screen_name set, if not then output error
			if ( !$settings['screen_name'] ) {
				echo '<blockquote class="tweet">Please set your twitter username in the settings panel.</blockquote>';
				return false;
			}
			
			if ($cache) {
				$output = stripslashes( $settings['html_template'] );
				$output = str_replace(
					array(
						'%screen_name%', '%followers%', '%tweet_id%', '%tweet_raw%', '%tweet%', '%created_at%',
						'%rel_time%'
					),
					array(
						$cache['screen_name'], number_format( $cache['followers'] ), $cache['tweet_id'], $cache['tweet_raw'], $cache['tweet'], $cache['created_at'],
						$this->relative_time( $cache['created_at'] )
					),
					$output
				);
				if ( !$output_results ) return $output;
				else echo $output;
				return true;
			}

			# No response from twitter and nothing in the cache
			if ( $settings['debug'] ) echo 'Sorry, no response from twitter and no cached data available.';
			return false;
		}
		
		/* settings page */
		function setup_settings_page() {
			if ( function_exists('add_options_page') ) {
				add_options_page( 'Latest Tweet Settings', 'Latest Tweet', 8, basename(__FILE__), array(&$this, 'print_settings_page') );
			}
		}
		function register_settings() {
			register_setting( 'sftweet-settings', 'latest_tweet_settings' );
			register_setting( 'sftweet-cache', 'latest_tweet_cache' );
		}
		function print_settings_page() {
			$settings = $this->get_settings();
			if ( isset($_POST['save_latest_tweet_settings']) ) {
				# Check if we need to reschedule updates
				$reschedule = ( $settings['cache_update'] == $_POST['latest_tweet_cache_update'] ) ? false : true;
				
				# Save new settings
				# echo 'Old: <code>'; print_r($settings); echo '</code>';
				foreach ( $settings as $name => $value ) {
					if ( $name!='last_updated' )
						$settings[$name] = $_POST['latest_tweet_'.$name];
				}
				# echo 'New: <code>'; print_r($settings); echo '</code>';
				update_option( 'latest_tweet_settings', $settings );
				
				# Reschedule update if needed
				if ( $reschedule ) $this->reschedule_updates();

				# Done!
				echo '<div class="updated"><p>Latest Tweet settings saved!</p></div>';

			} elseif ( isset($_POST['latest_tweet_UpdateCache']) ) {
				# Trigger update of tweets manually
				$this->cache_tweets();
				echo '<div class="updated"><p>Cache updated!</p></div>';

			} elseif ( isset($_POST['latest_tweet_ClearCache']) ) {
				# Empty cache to remove any extraneous data (?and trigger new update?)
				$this->empty_cache();
				echo '<div class="updated"><p>Cache cleared! You may want to manually update the cache now.</p></div>';

			} elseif ( isset($_POST['latest_tweet_ResetSchedule']) ) {
				$this->reschedule_updates();
				echo '<div class="updated"><p>Scheduled updates reset!</p></div>';

			} elseif ( isset($_POST['latest_tweet_ResetPlugin']) ) {
				# Reset Plugin
				delete_option('latest_tweet_settings');
				delete_option('latest_tweet_cache');
				echo '<div class="updated"><p>Latest Tweet plugin settings and cache reset to default!</p></div>';
			}
			
			# Pull settings again to account for updates before displaying form
			$settings = $this->get_settings();
			include( 'latest-tweet-settings.php' );
		}
		
		/* cache functions */
		function output_cache() {
			echo '<pre style="font-size:9px">';
			print_r( $this->get_cache() );
			echo '</pre>';
		}
		function empty_cache() {
			# Set up default
			$cache = array();

			# Overwrite cached data
			update_option( 'latest_tweet_cache', $cache );
		}
		
		/* settings link for plugin page */
		function add_actions( $links ) {
			array_unshift( $links, '<a href="options-general.php?page=latest-tweet.php">' . __('Settings') . '</a>' );
			return $links;
		}
		
		/* scheduler functions via wp_cron */
		function update_tweets() {
			$this->cache_tweets();
		}
		function reschedule_updates() {
			$settings = $this->get_settings();

			# Remove old schedule
			wp_clear_scheduled_hook( 'latest_tweet_scheduled_update' );
			
			# Set up new schedule, first run in 1hr (no scheduling for manual)
			if ( $settings['cache_update']!='manual' )
				wp_schedule_event( time()+3600, $settings['cache_update'], 'latest_tweet_scheduled_update' );
		}
		function activation() {
			$settings = $this->get_settings();

			# Populate initial cache, will only work if username has been previously configured
			$this->cache_tweets();

			# Set up scheduling, first batch in 1hr
			wp_schedule_event( time()+3600, $settings['cache_update'], 'latest_tweet_scheduled_update' );
		}
		function deactivation() {
			# wipe out cache, no need to waste space
			# but save settings in the event plugin is reactivated later
			update_option( 'latest_tweet_cache' );
			
			# clear scheduled update process
			wp_clear_scheduled_hook( 'latest_tweet_scheduled_update' );
		}
		
	} /* end:class */
} /* endif:class_exists */

$latestTweet = new latest_tweet();
add_action( 'admin_menu', array(&$latestTweet, 'setup_settings_page') );
add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), array(&$latestTweet, 'add_actions') );
register_activation_hook( __FILE__, array( &$latestTweet, 'activation' ) );
register_deactivation_hook( __FILE__, array( &$latestTweet, 'deactivation' ) );
add_action( 'latest_tweet_scheduled_update', array( &$latestTweet, 'update_tweets' ) ); #, priority, num args

function get_latest_tweet( $settings=array(), $output = true ) {
	global $latestTweet;
	$latestTweet->print_tweet( $settings, $output );
}